	Program TS_Steady_State_Feb_27_2020

        IMPLICIT NONE

        Include 'declare.inc'

        Double precision Vf(No), Vr(No), Em(No), Ep(No), Ets(No)
        Double precision Emax, QT

!!!  Em, Ep, Ets        : a vector of energies of minima, products, TSs
!!!  Vf, Vr             : a vector of forward and reverse barriers

        Integer MAXJ, NJ, NJJ

!!!  MAXJ       : maximum angular momentum number
!!!  NJ         : step size of angular momentum

        Double precision SYMRE(No), SYMTS(No), tp

        Double precision CON2, CON3 
        Parameter (CON2=4.184d0, CON3=1.0d0)

!!!  CON2       : a convert factor from kcal to kJ

        Integer  Nsink(No), IL(No), KL(No)  

!!!  Nsink      : a vector of sinks to collect stabilized intermediates 
!!!  IL, KL     : a vector of indexes of minima 

        Integer I, J, K, Maxi, KK, LL
        Integer Jmin, Jmax, Np
        Integer  NNNN, MMMM

!!!  Maxi       : maximum index used in master-equation 
!!!  NNNN       : angular momentum number (J) to be inputed.
!!!  MMMM       : pressure in reactor to be inputed.

        Integer NE(No+NM), NN, NTS, IND

!!!  NN         : a number of intermediates
!!!  NTS        : a number of TSs
!!!  IND        : index of the lowest-lying intermediate.

!!!        Double precision Sigf(No), Sigr(No)

!!!  Sigf, Sigr : vector of forward and reverse reaction path degeneracies.

        Double precision, Allocatable :: SS(:,:), TSS(:,:)
        Double precision, Allocatable :: DS(:,:), TDS(:,:)
        Double precision, Allocatable :: T1(:), AT(:)
        Double precision, Allocatable :: SS1(:), DS1(:)
        Double precision, Allocatable :: FE(:,:)
        Double precision, Allocatable :: SS0(:), DS0(:)


!!!  SS, TSS    : vector of sum and number of states for TSs
!!!  DS, TDS    : vector of sum and number of states for minima
!!!  FE         : initial energy distribution function, F(E,J)

        Double precision BT(3), BAJ

	Double precision CF, PP, beta, T, rKp, RKc, DENS
        Double precision E0, ENG, Edown, DEJ, DELE, AE, AJ 

!!!  Edown      : down energy transfer (cm-1)
!!!  DELE       : bin energy (grain energy) (cm-1)
!!!  RKc        : overall bi-molecular reaction (cm3/molecule/s) 
!!!  rKp        : collision frequency (1/s) at a particular pressure, PP (atm) 
!!!  beta       : = 1/RT (cm-1)
!!!  T          : temperature (K)
!!!  PP         : individual pressure (Torr)
!!!  CF         : collision frequency (1/s) at input pressure, Press (atm)

        Double precision su, suJ, suk 

        Double precision Press, m1, Dc1, xma1, m2, Dc2, xma2
        Double precision PMMMM

!!!  Press      : pressure (atm); often set Press = 1 atm
!!!  m1, m2     : masses of bath gas and intermediates/nascent adducts (g/mol)
!!!  Dc1, xma1  : Lennard-Jones collision parameters for bath gas: Sigma (Angstrom) and Eps/kB (K)
!!!  Dc2, xma2  : Lennard-Jones collision parameters for intermediates/nascent
!adducts

        CHARACTER*10 CHA 
        CHARACTER*27 CHAC1 
        CHARACTER*17 CHAC2
        CHARACTER*20 CNAME(NM,No)
        CHARACTER*20 CNM(No), CNP(No), CNTS(No)
        CHARACTER*20 CNMabc(No), CNTSabc(No)
        CHARACTER*3 FEINT, FEDISS
        CHARACTER*5 ESOLVER, ENTRANCETS
        CHARACTER*12 CDSM(No), CDSTS(No)

        Integer NDISS, II
        Integer NDI(NM), NDJ(No)

        Integer NDOF, MDOF, MX, MY
        Double precision FREQ(200) 
        Double precision BL, UL, RC(1000), EC(1000), Vff, Vrr
        Double precision Step, Eint, IFREQ, SFR
        CHARACTER*3 CFR
        Double precision, Allocatable :: FW(:,:)        
        Character*4 MODEL


!!!  CHA                : energy unit of minima, products, and TSs (kJ, or kcal, or cm-1)

!!!  CNM, CNP, CNTS     : names of minima, products, and TSs, which contain
!density and sum of states.
!!!  CNMabc, CNTSabc    : names of files that contain three rot. constants.
!!!  FEINT              : either "YES" or "NO"
!!!  ESOLVER            : either "DENSM" or "SPASM"
!!!  FEDISS             : either "YES" or "NO"
!!!  NDISS              : a number of F(E)J energy distribution fucntions at the
!moment of dissociation

        read(100,*) PMMMM   !!! Considered pressure in Torr
        read(101,*) NNNN   !!! Considered angular momentum number, J

        open (unit=1, file='TS.INP',status='old')

!!!  TS.INP   : main input file

        Read(1,*) Emax, DELE            !!! Line 1 : INPUT DATA
        Maxi = INT(Emax/DELE) + 1

        Read(1,*) MAXJ, NJ           !!! Line 2 : INPUT DATA

        Read(1,*) T, Press, Edown        !!! Line 3 : INPUT DATA
        Read(1,*) m1, Dc1, xma1         !!! Line 4 : INPUT DATA
        Read(1,*) m2, Dc2, xma2         !!! Line 5 : INPUT DATA
        Read(1,*) RKc, MODEL                   !!! Line 6 : INPUT DATA

        write(*,*) ""
        write(*,*) ""
        write(*,*) "Conditions (T,P,J) in Chemical Reactor : "
        write(*,*) "Temperature (K)         = ", REAL(T)
        write(*,*) "Pressure (Torr)         = ", REAL(PMMMM) 
        write(*,*) "Angular momentum number = ", REAL(NNNN)
        write(*,*) ""
        write(*,*) ""

!!!  To compute a Collision Frequency, CF

        CALL CalFC(CF,T,Press,m1,Dc1,xma1,m2,Dc2,xma2,MODEL) 

        CHAC1 = "Collision frequency (1/s)="
        CHAC2 = ", calculated at:"

        write(*,991) CHAC1, CF, CHAC2, Press, "atm & ", T, "K" 

991     FORMAT(A,1X,ES12.4,1X,A,1X,F10.2,1X,A,F10.2,1X,A)

        beta=1.D0/(RT*T)

        PP=PMMMM/760.0d0  !!! convert Torr to atm
        rKp = CF*PP            !!! CF at a considered pressure, PP 

        ALLOCATE ( T1(Maxi), AT(Maxi) )
        ALLOCATE ( SS1(Maxi), DS1(Maxi) )

        ALLOCATE ( SS0(Maxi), DS0(Maxi) )

        ALLOCATE( FE(Maxi,0:MAXJ) )


!!!!!!!!!!!!
!!!!!!!!!!!!
!! INPUT for Minima

        Read(1,*) NN, Np, CHA           !!! Line 7 : INPUT DATA

        ALLOCATE ( DS(NN,Maxi), TDS(NN,Maxi) )

        DO I=1, NN
                Read(1,*) J, CDSM(I), CNM(I), CNMabc(I), Em(I), SYMRE(I), tp !!! Line 8 : INPUT DATA
                SYMRE(I) = SYMRE(I)/tp

                IF(CHA.EQ.'kJ') THEN
                        Em(I) = Em(I)*con/CON2
                ELSE IF(CHA.EQ.'kcal') THEN
                        Em(I) = Em(I)*con
                ELSE IF(CHA.EQ.'cm-1') THEN
                        Em(I) = Em(I)*CON3
                ELSE
                        write(*,*) "WRONG INPUT in ENERGY UNIT"
                        STOP
                ENDIF
        ENDDO
        DO I=1, Np
                Read(1,*) J, CNP(I), Ep(I+NN)              !!! Line 9 : INPUT
                IF(CHA.EQ.'kJ') THEN
                        Ep(I+NN) = Ep(I+NN)*con/CON2
                ELSE IF(CHA.EQ.'kcal') THEN
                        Ep(I+NN) = Ep(I+NN)*con
                ELSE IF(CHA.EQ.'cm-1') THEN
                        Ep(I+NN) = Ep(I+NN)*CON3
                ELSE
                        write(*,*) "WRONG INPUT in ENERGY UNIT"
                        STOP
                ENDIF
        ENDDO

        DO I=1, NN
                DO J=1, Np+1
                IF(J.eq.1) THEN
                        CNAME(I,J) = CNM(I)
                ELSE
                        CNAME(I,J) = CNP(J-1)
                ENDIF
                ENDDO
        ENDDO 

        E0 = Em(1)
        IND = 1
        DO I=2, NN
                IF(E0.GT.Em(I)) THEN
                        E0 = Em(I)
                        IND = I
                ENDIF
        ENDDO

        write(*,*) ""
        write(*,*) "Minimum Index and Its Position in Energy-Grained ME:"
        DO I=1, NN
                ENG = Em(I) - E0
                NE(I) = NINT(ENG/DELE) + 1
                write(*,*) I, NE(I)
        ENDDO
        write(*,*) ""

        DO I=1, NN

        IF(CDSM(I).EQ.'EXTERNAL') THEN

        open (unit=10+I, file=CNM(I),status='old')
                DO J=1, Maxi
                        Read(10+I,*) K, ENG, DENS, DS(I,J) !!! INPUT : dens/sums 
                ENDDO
        close (unit=10+I, status='keep')

        ELSE IF(CDSM(I).EQ.'CRRKM_HO') THEN
        open (unit=10+I, file=CNM(I),status='old')
                Read(10+I,*) NDOF
                DO J=1, NDOF
                        Read(10+I,*) FREQ(J)
                ENDDO
        close (unit=10+I, status='keep')                         
      
                CALL driver_CRRKM( SS0, DS0, Maxi, DELE, NDOF, FREQ ) 
                DO J=1, Maxi
                        DS(I,J) = SS0(J)
                ENDDO
        ELSE
                write(*,*) "INPUT DATA WRONG for INTERMEDIATE ", I
        ENDIF 
 
        open (unit=1000+I, file=CNMabc(I),status='old')
                DO J=1, 3
                        Read(1000+I,*) BB(I,J)  !!! INPUT : three rot. constants
                ENDDO
        close (unit=1000+I, status='keep') 
        ENDDO

        DO I=1, NN
                DO J=1, Maxi
                        IF(J.LT.NE(I)) THEN
                                TDS(I,J) = 0.0d0
                        ELSE IF(J.EQ.NE(I)) THEN
                             TDS(I,J) = DS(I,J-NE(I)+1)
                        ELSE
                             TDS(I,J) = DS(I,J-NE(I)+1) - DS(I,J-NE(I))
                        ENDIF
                ENDDO
!!!!                IF(NE(I).EQ.1) TDS(I,1) = 1.0d0  !!! ????

!!!!!!!!!  
!!!   Set SINK for each stable INTERMEDIATE
!!!!!!!!!


                Nsink(I) = NE(I) + NINT(RT*T/DELE)  

!!        Nsink(I) = NE(I) + NINT(2*RT*T/DELE) 

        ENDDO

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!
!! J-effect on <DE>d = <DE>d_vib + <DE>d_rot
!! <DE>d_rot = <B> x Del_J x (2*J + 1 - Del_J)
!! Spherical top
!!
        BAJ = 1.0d0
        DO I=1, 3
                BAJ = BAJ * BB(IND,I)
        ENDDO
        BAJ = (BAJ)**(1.0d0/3)

        Edown = Edown + BAJ*NJJ*(2*NNNN+1-NJJ)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



!! INPUT for TSs

        Read(1,*) NTS, CHA              !!! Line 10 : INPUT 

        ALLOCATE ( SS(NTS,Maxi), TSS(NTS,Maxi) )


        DO I=1, NTS
        Read(1,*) J,CDSTS(I),CNTS(I),CNTSabc(I),IL(I),KL(I),ETS(I),SYMTS(I),tp !!! Line 11 : INPUT

        SYMTS(I) = SYMTS(I)/tp

                IF(CHA.EQ.'kJ') THEN
                        ETS(I) = ETS(I)*con/CON2
                ELSE IF(CHA.EQ.'kcal') THEN
                        ETS(I) = ETS(I)*con
                ELSE IF(CHA.EQ.'cm-1') THEN
                        ETS(I) = ETS(I)*CON3
                ELSE
                        write(*,*) "WRONG INPUT in ENERGY UNIT"
                        STOP
                ENDIF

                Vf(I) = ETS(I) - Em(IL(I))
                IF(KL(I).GT.NN) THEN
                        Vr(I) = ETS(I) - Ep(KL(I))
                ELSE
                        Vr(I) = ETS(I) - Em(KL(I))
                ENDIF

        write(*,*) I, Vf(I)*CON2/con, Vr(I)*CON2/con

        ENDDO


        DO I=1, NTS

        IF(CDSTS(I).EQ.'EXTERNAL') THEN

        open (unit=10+I+NN, file=CNTS(I),status='old')
                DO J=1, Maxi   !!! INPUT : dens/sums for TSs
                        Read(10+I+NN,*) K, ENG, DENS, SS(I,J) 
                ENDDO
        close (unit=10+I+NN, status='keep')

        ELSE IF(CDSTS(I).EQ.'CRRKM_HO') THEN
        open (unit=10+I+NN, file=CNTS(I),status='old')
                Read(10+I+NN,*) NDOF
                DO J=1, NDOF
                        Read(10+I+NN,*) FREQ(J)
                ENDDO
        close (unit=10+I+NN, status='keep')

                CALL driver_CRRKM( SS0, DS0, Maxi, DELE, NDOF, FREQ )       
                DO J=1, Maxi
                        SS(I,J) = SS0(J)
                ENDDO
        ELSE IF(CDSTS(I).EQ.'VRRKM_HO') THEN

        open (unit=10+I+NN, file=CNTS(I),status='old')
                Read(10+I+NN,*) MX, BL, UL 
                Read(10+I+NN,*) MY, Step, Eint, CHA
                Read(10+I+NN,*) CFR, SFR

                ALLOCATE ( FW(MX,MY+4) )

                DO J=1, MY + 4 
                        Read(10+I+NN,*) (FW(K,J), K=1, MX)
                ENDDO

        IF(CHA.EQ.'kJ') THEN       !!! convert to kcal from kJ
                Eint = Eint/CON2
                DO K=1, MX
                        FW(K,MY+4)=FW(K,MY+4)/CON2
                ENDDO
        ELSE IF(CHA.EQ.'kcal') THEN   !!! kcal to kcal
                Eint = Eint/CON3
                DO K=1, MX
                        FW(K,MY+4)=FW(K,MY+4)/CON3
                ENDDO
        ELSE IF(CHA.EQ.'cm-1') THEN  !!! cm-1 to kcal
                Eint = Eint/con
                DO K=1, MX
                        FW(K,MY+4)=FW(K,MY+4)/con
                ENDDO
        ELSE
                write(*,*) "UNIT of ENERGY INPUT is WRONG"
                STOP
        ENDIF

        close (unit=10+I+NN, status='keep')

                CALL driver_VRRKM( SS0, MX, MY, Maxi, DELE, NNNN, &
                        BL, UL, Step, Eint, E0/con, FW, CFR, SFR )

                TSS(I,1) = SS0(1)
                write(5000+I,*) TSS(I,1)
                DO J=2, Maxi
                        TSS(I,J) = SS0(J) - SS0(J-1)
                        write(5000+I,*) TSS(I,J)
                ENDDO

                IF(I.EQ.1) THEN
                QT=0.0d0
                DO J=0, MAXJ, NJ
                CALL driver_VRRKM( SS0, MX, MY, Maxi, DELE, J, &
                        BL, UL, Step, Eint, E0/con, FW, CFR, SFR )
                        DO II=1, Maxi
                        ENG = (II-1)*DELE
                        FE(II,J) = (2*J+1)*SS0(II)*dexp(-beta*ENG) 
                        QT = QT + FE(II,J)
                        ENDDO
                ENDDO
                QT = QT*DELE*NJ*beta*dexp(-E0*beta)

                write(10000,1111) T, QT

1111    FORMAT(F10.2,2X,ES15.5)

                ENDIF

                DEALLOCATE ( FW )


        ELSE IF(CDSTS(I).EQ.'QRRKM_Eckart') THEN
 
        open (unit=10+I+NN, file=CNTS(I),status='old')

                Read(10+I+NN,*) NDOF, IFREQ
                DO J=1, NDOF
                        Read(10+I+NN,*) FREQ(J)   !!! I and J here
                ENDDO
        close (unit=10+I+NN, status='keep')
        CALL driver_QRRKM_Eckart( SS0, DS0, Vf(I), Vr(I), Maxi, DELE, NDOF, FREQ, IFREQ ) 
                DO J=1, Maxi
                        SS(I,J) = SS0(J)
                ENDDO

        ELSE IF(CDSTS(I).EQ.'QRRKM_WKB') THEN

        open (unit=10+I+NN, file=CNTS(I),status='old')
                Read(10+I+NN,*) NDOF, MDOF
                DO J=1, NDOF
                        Read(10+I+NN,*) RC(J), EC(J)
                ENDDO
        close (unit=10+I+NN, status='keep')

        Vff = Vf(I) / 349.754d0 / 627.5095d0
        Vrr = Vr(I) / 349.754d0 / 627.5095d0

        CALL driver_QRRKM_WKB( SS0, DS0, Vff, Vrr, Maxi, DELE, NDOF, FREQ, MDOF, &
                RC, EC ) 

                DO J=1, Maxi
                        SS(I,J) = SS0(J)
                ENDDO

        ELSE
                write(*,*) "INPUT DATA WRONG"
        ENDIF

!!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        open (unit=1000+I+NN, file=CNTSabc(I),status='old')
                DO J=1, 3     !!! INPUT : three rot. constants for TSs
                        Read(1000+I+NN,*) BBT(I,J)
                ENDDO
        close (unit=1000+I+NN, status='keep')
        ENDDO

        DO I=1, NTS
                IF(SS(I,1).EQ.1.0d0) THEN
                        ENG = ETS(I) - E0
                        NE(I+NN) = NINT( ENG/DELE ) + 1
                ELSE IF(Vf(I).GT.Vr(I)) THEN
                        ENG = ETS(I) - Vr(I) - E0
                        NE(I+NN) = NINT( ENG/DELE ) + 1
                ELSE
                        ENG = ETS(I) - Vf(I) - E0
                        NE(I+NN) = NINT( ENG/DELE) + 1
                ENDIF
        ENDDO

        DO I=1, NTS
                IF(CDSTS(I).NE.'VRRKM_HO') THEN

                DO J=1, Maxi
                        IF(J.LT.NE(I+NN)) THEN
                                TSS(I,J) = 0.0d0
                        ELSE IF(J.EQ.NE(I+NN)) THEN
                                TSS(I,J) = SS(I,J-NE(I+NN)+1)
                        ELSE
                                TSS(I,J) = SS(I,J-NE(I+NN)+1) - SS(I,J-NE(I+NN))
                        ENDIF
                ENDDO
!!!!                IF(NE(I+NN).EQ.1) TSS(I,1) = 1.0d0  !!! ????
                ENDIF
        ENDDO

!!!!!!!!!!!!
!!!!!!!!!!!!

        Read(1,*) ESOLVER               !!! Line 12 : INPUT DATA
        Read(1,*) FEINT, ENTRANCETS                 !!! Line 13 : INPUT DATA
        Read(1,*) FEDISS, NDISS         !!! Line 14 : INPUT DATA
        IF(FEDISS.EQ.'YES') THEN
                DO I=1, NDISS
                        Read(1,*) J, NDI(I), NDJ(I)  !!! Line 15 : INPUT DATA
                ENDDO
        ENDIF

        close (unit=1, status='keep')

!!        ALLOCATE( FE(Maxi,0:MAXJ) )

!!CCCCCCCCCCCCCCCCCCCCCCCC

        IF(FEINT.EQ.'NO') THEN  !!! To compute F(E,J)

!!
!! TS1 is assumed to be an entrance TS to build F(E,J)
!!

        IF(ENTRANCETS.EQ.'TIGHT') THEN    !!! For a Tight TS

        DO I=1, Maxi
                T1(I) = TSS(1,I)
        ENDDO
        DO I=1, 3
                BT(I) = BBT(1,I)
        ENDDO

!!CCCCCCCCCCCCCCCCCCCCCCC

        AJ = 0.0d0
        suJ = 0.0d0
        AE = 0.0d0

        DO J=0, MAXJ, NJ

        CALL cdsj(SS1,DS1,T1,AT,DELE,Maxi,BT,J)

        su=0.0D0
        Do I=1,Maxi
         ENG = (I-1)*DELE
!!         IF(I.LT.NE(NN+1)) THEN
!!             FE(I,J) = 0.0d0
!!         ELSE
             FE(I,J) = (2*J+1)*SS1(I)*dexp(-beta*ENG)
!!         ENDIF
             su = su + FE(I,J)
             AE = AE + ENG*FE(I,J)
        Enddo
        AJ = AJ + su*J
        suJ = suJ + su
        ENDDO

        AJ = AJ/suJ
        AE = AE/suJ

         write(*,*) ""
         write(*,*) "Expectation values from F(E,J) : " 
         write(*,*) "Average J        = ", REAL(AJ)
         write(*,*) "Average E (cm-1) = ", REAL(AE)
         write(*,*) ""

        ELSE IF(ENTRANCETS.EQ.'LOOSE') THEN   !!! For a Loose TS 

        AJ = 0.0d0
        suJ = 0.0d0

        DO J=0, MAXJ, NJ

        su=0.0D0
        Do I=1,Maxi
             ENG = (I-1)*DELE
             su = su + FE(I,J)
        Enddo
        AJ = AJ + su*J
        suJ = suJ + su
        ENDDO

        AJ = AJ/suJ

        AE = 0.0d0
        DO I=1, Maxi
                ENG = (I-1)*DELE
                su = 0.0d0
                DO J=0, MAXJ, NJ
                        su = su + FE(I,J)
                ENDDO
                AE = AE + ENG*su
        ENDDO

        AE = AE/suJ

         write(*,*) ""
         write(*,*) "Expectation values from F(E,J) : "
         write(*,*) "Average J        = ", REAL(AJ)
         write(*,*) "Average E (cm-1) = ", REAL(AE)
         write(*,*) ""

!!CCCCCCCCCCCCCCCCCCC

!!        J = NNNN
!!        suJ = 0.0d0
!!        DO I=1, Maxi
!!                ENG = (I-1)*DELE
!!                suJ = suJ + FE(I,J)
!!        ENDDO
       
!!        DO I=1, Maxi
!!                FE(I,J) = FE(I,J)/suJ
!!        ENDDO 
!!        suJ = 0.0d0
!!        DO I=1, Maxi
!!                suJ = suJ + FE(I,J)
!!        ENDDO

        ENDIF

        ELSE IF(FEINT.EQ.'YES') THEN  !!! F(E)J from an EXTERNAL FILE

                suJ = 0.0d0
                open (unit=2, file='FE.EXTERNAL',status='old')
                        DO I=1, Maxi
                                Read(2,*) J, ENG, FE(I,NNNN)
                                suJ = suJ + FE(I,NNNN)
                        ENDDO
                close (unit=2, status='keep')
        ELSE
                write(*,*) "SOMETHING WRONG with INPUT DATA"
                STOP

        ENDIF

        write(333,'(I5,1X,I5,1X,I5)') INT(MAXJ/NJ)+1, NN, Np

        Jmin = NNNN
        Jmax = NNNN

!!! To compute thermal rate constants (or yields) for each product:

       CALL CALRATE(Jmin,Jmax,MAXJ,NJ,DELE,Maxi,FE,NN,NTS,Np,IND,Edown,T,Nsink,suJ,rKp,RKc,&
                ESOLVER, BB, BBT, TSS, TDS, CNAME, CNM, CNP, IL, KL,&
                FEDISS, NDISS, NDI, NDJ, CDSTS, SYMRE, SYMTS )

        DEALLOCATE ( T1, AT, SS1, DS1 )
        DEALLOCATE ( SS, DS, TSS, TDS )
        DEALLOCATE ( FE )
        STOP
	END Program TS_Steady_State_Feb_27_2020 



