	SUBROUTINE driver_QRRKM_Eckart( SS, DS, V1, V2, JMAX, DELE, N, FREQ, IFREQ ) 

	IMPLICIT NONE

!! JMAX = Maxinum naumber of bins in master-equation
!! DELE = Energy bin
!! N = Number of Vibrational Degrees of Freedom (DOF) 
!! FREQ = Vector of harmonic vibration frequencies with N elements
!! SS (OUTPUT) = Vector of Sum of vibrational states 
!! DS (OUTPUT) = Vector of Density of vibrational states
!! V1 : Forward barrier (cm-1)
!! V2 : Reverse barrier (cm-1)
!! IFREQ : Magnitude of Imaginary Frequency

	Integer I, J,  N
	Integer JMAX
        Double precision DELE, IFREQ, ENG
	Double precision V1, V2, V1b, V2b 
	Double precision T(JMAX), AT(JMAX)
	Double precision SS(JMAX), DS(JMAX)
	Double precision FREQ(N)
	Double precision PE 

	IF(V1.GT.V2) THEN
		V1b = V2
		V2b = V1
	ELSE
		V1b = V1
		V2b = V2
	ENDIF

	DO I=1, JMAX
		ENG = (I-1)*DELE
		AT(I) = 0.0d0
		T(I) = PE(ENG,V1b,V2b,IFREQ)
	ENDDO

	DO I=1, N
		CALL HO(T,AT,DELE,JMAX,FREQ(I))
	ENDDO

	DS(1) = T(1)/DELE
	SS(1) = T(1)
	DO I=2, JMAX
		DS(I) = (T(I)-T(I-1))/DELE
		SS(I) = T(I)
	ENDDO

	RETURN
	END SUBROUTINE driver_QRRKM_Eckart

!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        DOUBLE PRECISION FUNCTION PE(eng,V1,V2,ifreq)
	IMPLICIT NONE
        External fund,funa,funb
        Double precision eng,V1,V2,ifreq
        Double precision tp1,tp2,tp3, P2
        Double precision fund,funa,funb


                tp1=funa(eng,V1,V2,ifreq)
                tp2=funb(eng,V1,V2,ifreq)
                tp3=fund(V1,V2,ifreq)
                PE=1.0D0-(cosh(tp1-tp2)+tp3)/(cosh(tp1+tp2)+tp3)

!CC                IF(eng.LE.V1) THEN
!CC                        tp1=funa(eng,V1,V2,ifreq)
!CC                        tp2=funb(eng,V1,V2,ifreq)
!CC                        tp3=fund(V1,V2,ifreq)
!CC                        PE=1.0D0-(cosh(tp1-tp2)+tp3)/(cosh(tp1+tp2)+tp3)
!CC                ELSE IF((eng.GT.V1).and.(eng.LE.V1*2)) THEN
!CC                        tp1=funa(V1*2-eng,V1,V2,ifreq)
!CC                        tp2=funb(V1*2-eng,V1,V2,ifreq)
!CC                        tp3=fund(V1,V2,ifreq)
!CC                        PE=(cosh(tp1-tp2)+tp3)/(cosh(tp1+tp2)+tp3)
!CC                ELSE
!CC                        PE=1.0D0
!CC                ENDIF

        RETURN
        END

        Double precision FUNCTION fund(V1,V2,ifreq)
        IMPLICIT NONE
        Double precision V1,V2,tp,pi,ifreq,z1,z2
        pi=acos(-1.0D0)
        z1=2.0D0*pi*V1/ifreq
        z2=2.0D0*pi*V2/ifreq
        tp=z1*z2-4.0D0*pi*pi/16
        if(tp.ge.0.) then
                fund=cosh(2*sqrt(tp))
        else
                fund=cos(2*sqrt(abs(tp)))
        endif
        RETURN
        END

        DOUBLE PRECISION FUNCTION funb(eng,V1,V2,ifreq)
        IMPLICIT NONE
        Double precision eng,V1,V2,ifreq,z1,z2,z3,tp1,tp2,pi
        pi=acos(-1.0D0)
        z1=2.0D0*pi*V1/ifreq
        z2=2.0D0*pi*V2/ifreq
        z3=eng/V1
        tp1=z3*z1+z2-z1
        tp2=1.0D0/sqrt(z1)+1.0D0/sqrt(z2)
        funb=2.0D0*sqrt(tp1)/tp2
        Return
        END

        DOUBLE PRECISION FUNCTION funa(eng,V1,V2,ifreq)
        IMPLICIT NONE
        Double precision eng,V1,V2,ifreq,z1,z2,z3,tp1,tp2,pi
        pi=acos(-1.0D0)
        z1=2.0D0*pi*V1/ifreq
        z2=2.0D0*pi*V2/ifreq
        z3=eng/V1
        tp1=z3*z1
        tp2=1.0D0/sqrt(z1)+1.0D0/sqrt(z2)
        funa=2.0D0*sqrt(tp1)/tp2
        Return
        END





	
