	SUBROUTINE driver_QRRKM_WKB( SS, DS, V1, V2, JMAX, DELE, N, FREQ, M, &
		RC, EC ) 

	IMPLICIT NONE

	Double precision CON, CC, DX
	Parameter ( CON = 349.75479275231D0, CC = 627.5095D0, &
		DX = 1.0D-4 )

!! JMAX = Maxinum number of bins in master-equation
!! DELE = Energy bin
!! N = Number of Vibrational Degrees of Freedom (DOF) 
!! FREQ = Vector of harmonic vibration frequencies with N elements
!! SS (OUTPUT) = Vector of Sum of vibrational states 
!! DS (OUTPUT) = Vector of Density of vibrational states
!! V1 : Forward barrier (Hartree)
!! V2 : Reverse barrier (Hartree)
!! M : Number of grid points along the Reaction Coordinate
!! RC : Vector of grid points along the Reaction Coordinate (Bohr x amu^1/2)
!! EC : Vector of RELATIVE ENERGIES to TS, along the Reaction Coordinate (Hartree) 
!! DX : Step size in computing INTEGRAL (Bohr x amu^1/2)

	Integer I, J,  N, M
	Integer JMAX
        Double precision DELE, ENG
	Double precision V1, V2, V1b, V2b 
	Double precision T(JMAX), AT(JMAX)
	Double precision SS(JMAX), DS(JMAX)
	Double precision FREQ(N)
	Double precision tp, PE 
	Double precision RC(M), EC(M)
	Double precision X(M), Y(M), Y2(M)
	Double precision X0, V0

	IF(V1.GT.V2) THEN
		V1b = V2
		V2b = V1
	ELSE
		V1b = V1
		V2b = V2
	ENDIF

        DO I=1, M
                X(I) = RC(I)
                Y(I) = EC(I) + V1b
	ENDDO 

	tp = Y(1)*Y(M)
	IF(tp.LE.0.0d0) THEN
		IF(Y(1).GE.0.0d0) THEN
			X0 = X(1)
			V0 = Y(1)
		ELSE
			X0 = X(M)
			V0 = Y(M)
		ENDIF
	ELSE IF(Y(1).GE.Y(M)) THEN
		X0 = X(1)
		V0 = Y(1)
	ELSE IF(Y(1).LT.Y(M)) THEN
		X0 = X(M)
		V0 = Y(M)
	ENDIF

        CALL spline(X,Y,M,2.0d+30,2.0d+30,Y2)
!!	CALL splint(X,Y,Y2,M,X0,V0)

!! X0 = The lowest-lying turning point, below it PE is set to ZERO
!! V0 = The lowest-lying energy

	DO I=1, JMAX
		ENG = (I-1)*DELE
		ENG = ENG / CON / CC
		AT(I) = 0.0d0
		CALL cdP2(PE,ENG,V1b,DX,V0,X,Y,Y2,M) 
		T(I) = PE
	ENDDO

	DO I=1, N
		CALL HO(T,AT,DELE,JMAX,FREQ(I))
	ENDDO

	DS(1) = T(1)/DELE
	SS(1) = T(1)
	DO I=2, JMAX
		DS(I) = (T(I)-T(I-1))/DELE
		SS(I) = T(I)
	ENDDO

	RETURN
	END SUBROUTINE driver_QRRKM_WKB

!!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


      SUBROUTINE cdP2(P2,eng,bar,DX,V0,X,Y,Y2,M)
	IMPLICIT NONE 
	Integer M
	Double precision DX, X0, V0, tp
	Double Precision eng,bar,P1,P2
	Double precision X0n, X0p
	Double precision X(M), Y(M), Y2(M)

	IF(eng.eq.0.0D0) THEN
		P2=0.0D0
	ELSE
	If(eng.eq.bar) then
		P2=0.5D0
	Else if((eng.gt.bar).and.(eng.lt.(2.0D0*bar))) then
		P2=1.0D0 - P1(bar*2.0D0-eng,DX,V0,X,Y,Y2,M)
	Else if(eng.ge.(2.0D0*bar)) then
		P2=1.0D0
	else
		P2=P1(eng,DX,V0,X,Y,Y2,M)
	Endif
	ENDIF

	RETURN
	END

	Double Precision FUNCTION P1(eng,DX,V0,X,Y,Y2,M)
	IMPLICIT NONE
	Integer M 
	Double precision X0, V0, tp, XN, XP
	Double Precision eng,bar,ifreq,temp,V
	Double Precision kf,x1,x2,DX,h
	Double Precision temp2,temp3,pi,su,CalInt
	Double Precision X(M), Y(M), Y2(M)

	IF(eng.LT.V0) THEN
		P1=0.0d0
	ELSE

	pi=acos(-1.0D0)
	x1 = XN(eng,DX,X,Y,Y2,M)
	x2 = XP(eng,DX,X,Y,Y2,M)

		su = CalInt(x1,x2,eng,X,Y,Y2,M)
		su = su * 6.795159D0 * 2.0D0 * pi
		temp2=2.0D0 * su
		P1=1.0D0 / (1.0D0 + exp(temp2))
	ENDIF

	RETURN
	END 

	Double Precision FUNCTION CalInt(x1,x2,eng,XX,YY,YY2,M)
	IMPLICIT NONE
	Integer M
	Double precision accu 
	Parameter (accu=1.D+4)
	Double precision X0, tp, Vx
	Double Precision x,x1,x2,eng,bar,sum,dx,temp
	Double Precision XX(M), YY(M), YY2(M)

                dx=(x2-x1)/accu
                x=x1

		CALL splint(XX,YY,YY2,M,x,Vx)
!!		Vx = Vx/627.5095d0

		temp=abs(eng-Vx)
                sum=0.5D0 * sqrt(2.0D0*temp)
		x=x+dx

200             CALL splint(XX,YY,YY2,M,x,Vx)
!!		Vx = Vx/627.5095d0  !!! convert to hartree from kcal/mol
		temp=2.0D0*abs(eng-Vx)
                sum=sum+sqrt(temp)
		x=x+dx
                If(x.lt.x2) then
                        goto 200
                endif
		x=x2	

		CALL splint(XX,YY,YY2,M,x,Vx)
!!		Vx = Vx/627.5095d0

		temp=abs(eng-Vx)		

		sum=sum+0.5D0 * sqrt(2.0D0*temp)
		CalInt=sum*dx
	RETURN
	END

!!!!!!!!!!!!!!!!!

        DOUBLE PRECISION FUNCTION XN(E,DX,X,Y,Y2,M)
	IMPLICIT NONE
        Double precision  DX, E, X0, tp, X02, Vx 
	Integer M
	Double precision Y2(M), X(M), Y(M)

        X02 = 0.0d0 
993     CALL splint(X,Y,Y2,M,X02,Vx)   
!!	tp=Vx/627.5095d0

	tp = Vx

        IF(tp.GT.E) THEN
                X02 = X02 - DX    
                goto 993
        ENDIF
        
        XN=X02

        RETURN
        END

!!!!!!!!!!!!!!!!!

	DOUBLE PRECISION FUNCTION XP(E,DX,X,Y,Y2,M)
	IMPLICIT NONE
	Double precision  DX, E, X0, tp, X02, Vx 
	Integer M
	Double precision Y2(M), X(M), Y(M)

	X02 = 0.0d0 
994	CALL splint(X,Y,Y2,M,X02,Vx)	
!!	tp=Vx/627.5095d0

	tp = Vx

	IF(tp.GT.E) THEN
		X02 = X02 + DX	
		goto 994
	ENDIF
	
	XP=X02

	RETURN
	END



