	SUBROUTINE driver_VRRKM( GE, MX, MY, JMAX, DELE, JJ, &
		BL, UL, Step, Eto, E0, GW, CFR, SFR )

	IMPLICIT NONE

integer, parameter :: qp = selected_real_kind(15, 307)

!! MX = Number of Grid Points
!! MY = Number of harmonic frequencies (3N-7), projecting out the Reaction Coordinate
!! BL = Lower Limit of bond length 
!! UL = Upper Limit of bond length 
!! Eto = Energy of Reactant (kcal/mol)
!! E0 = Energy of the lowest-lying intermediate (kcal/mol)
!! Step = step size used to locate a variational bottleneck.
!! JMAX = maxinum number of bins in master-equation
!! DELE = energy bin
!! GE = vector of sum of states for the bottleneck
!! GW = matrix with a dimension of (MX x (MY+4)) that contains harmonic frequencies, rot. constants, and barrier heights for Grid Points

	real(kind=qp) :: CON
	Parameter ( CON = 349.75479275231D0 )

        real(kind=qp) ::  GW(MX,MY+4), GE(JMAX)

	Integer MX, MY 
	real(kind=qp) :: A(MX), X(MX), Y(MX), Y2(MX) 
	Integer I, J, K, NN
	Integer JMAX, Ntp, JJ
	real(kind=qp) :: XX, YY
	real(kind=qp) :: BL, UL, Step, Eto, Eb, E0
        real(kind=qp) :: DSS1, DELE, Eng, SFR, E1, E2
        real(kind=qp) :: BB(3), FREQ
	real(kind=qp) :: T(JMAX), AT(JMAX), T1(JMAX), T2(JMAX)
	real(kind=qp) :: SS(JMAX), DS(JMAX), SS1(JMAX)

	real(kind=qp), Allocatable :: GSS(:,:)
        real(kind=qp),  Allocatable :: GWF(:,:)

        CHARACTER*3 CFR

	DSS1 = (UL-BL)/(MX-1)

	NN = INT((UL-BL)/Step) + 1

	ALLOCATE ( GSS(NN,JMAX) )
	ALLOCATE ( GWF(NN,MY+4) )

	DO J=1, MY+4
		DO I=1, MX
			A(I) = GW(I,J)
			X(I) = BL + (I-1)*DSS1
		ENDDO
		CALL spline(X,A,MX,2.0D+30,2.0D+30,Y2)
		DO I=1, NN
			XX = BL + (I-1)*Step
			CALL splint(X,A,Y2,MX,XX,YY)
			GWF(I,J) = YY
		ENDDO
	ENDDO

	DO I=1, NN

        IF(CFR.EQ.'NO') THEN

		CALL TAT(T,AT,JMAX)
		DO J=1, MY
			FREQ = GWF(I,J)
			CALL HO(T,AT,DELE,JMAX,FREQ)
		ENDDO

        ELSE IF(CFR.EQ.'YES') THEN

                T(1) = 0.0D0
                AT(1) = 0.0D0
                DO K=2, JMAX
                        E1 = (K-1)*DELE
                        E2 = (K-2)*DELE 
                        T(K) = sqrt(E1/GWF(I,MY)) - sqrt(E2/GWF(I,MY))
                        T(K) = 2*T(K)/SFR
                        AT(K) = 0.0D0
                ENDDO
                DO J=1, MY-1
                        FREQ = GWF(I,J)
                        CALL HO(T,AT,DELE,JMAX,FREQ)
                ENDDO

        ELSE
                write(*,*) "INPUT FILE IS WRONG AT FREE-ROTOR"
                STOP
        ENDIF

		DO J=1, 3
			BB(J) = GWF(I,J+MY)
		ENDDO
		Eb = GWF(I,MY+4)

                Ntp = INT((Eto+Eb-E0)*CON/DELE) + 1  !!! 
	
		DO J=1, JMAX
			IF(J.LT.Ntp) THEN
				T2(J) = 0.0D0
			ELSE
				T2(J) = T(J-Ntp+1)
			ENDIF
		ENDDO

		CALL cdsj(SS,DS,T2,AT,DELE,JMAX,BB,JJ)

		DO J=1, JMAX
			GSS(I,J) = SS(J)
			Eng = (J-1)*DELE
			DS(J) = 0.0D0
			SS(J) = 0.0D0
		ENDDO
	ENDDO

	DO J=1, JMAX
		GE(J) = GSS(1,J)
	ENDDO

	DO J=1, JMAX
		DO I=2, NN
			IF(GE(J).GT.GSS(I,J)) GE(J) = GSS(I,J)
		ENDDO
	ENDDO

	DEALLOCATE ( GSS, GWF )

	RETURN
	END SUBROUTINE driver_VRRKM
	
