!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE CalA 
!CCC Purpose    : To build an A matrix in AX = B equation
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


	SUBROUTINE CalA(A,rKp,P,rktp,Maxi,KK,LL,NN,Np) 

!!!  A          : matrix A
!!!  rKp        : collision frequency (1/s) 
!!!  P          : energy transfer probability function
!!!  rktp       : microcanonical rate constants
!!!  Maxi       : maximum index used in master-equation
!!!  KK, LL     : index of intermediate
!!!  NN         : number of minima
!!!  Np         : number of products  


!!	IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)

        IMPLICIT NONE
        Integer I, J, Maxi, KK, LL, NN, Np
        Double precision rKp, A0

        Double precision A(NN*Maxi,NN*Maxi), P(NN,Maxi,Maxi), rktp(NN,NN+Np,Maxi)


	IF(KK.EQ.LL) THEN
		DO I=1, Maxi
			DO J=1, Maxi
				IF(I.EQ.J) THEN
			A0 = -rKp*P(KK,I,I) + rktp(KK,KK,I)
                IF(A0.NE.0.0d0) THEN
                        A0 = A0 + rKp
                ELSE
                        A0 = rKp 
                ENDIF
				ELSE
					A0=-rKp*P(KK,I,J)
				ENDIF
				A(I+Maxi*(KK-1),J+Maxi*(KK-1))=A0
			ENDDO
		ENDDO
	ELSE
		DO I=1, Maxi
			DO J=1, Maxi
				IF(I.EQ.J) THEN
					A(I+Maxi*(LL-1),J+Maxi*(KK-1))=-rktp(KK,LL,I)
				ELSE
					A(I+Maxi*(LL-1),J+Maxi*(KK-1))=0.0D0
				ENDIF
			ENDDO
		ENDDO
	ENDIF
	RETURN
	END
