!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : SUBROUTINE CalP
!CCC Purpose    : To compute energy transfer probability function 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        SUBROUTINE CalP(P,dE,AL,Maxi,T,Nsink,NN,w,IND)

!!!  P          : energy transfer probability function
!!!  dE         : energy bin (energy grain)
!!!  AL         : down energy (cm-1) 
!!!  Maxi       : maximum index in master-equation
!!!  T          : temperature
!!!  Nsink      : sink is set to collect stabilized populations of
!!!                     intermediates
!!!  NN         : number of intermediates
!!!  w          : density of states of considered intermediate
!!!  IND        : index of the lowest-lying intermediate

!!        IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)
  
        IMPLICIT NONE
      
        Integer I, J, N, Maxi, n1, K, Nsink(NN), NN, IND

        Double precision CN(Maxi), E(Maxi), w(Maxi), C(Maxi)

        Double precision P(NN,Maxi,Maxi)

        Double precision EI, EJ, dE, T, AL, beta, su
        Double precision su1, su2, tp, tpp1, tpp2, tpp3, tpp
        Double precision Pdij, Puij

        call CalCN(CN,w,dE,AL,Maxi,T,IND)

        DO K=1, NN
                DO J=1, Maxi
                        DO I=1, Maxi
                           IF(J.lt.Nsink(K)) THEN
                                      P(K,I,J)=0.0d0
                           ELSE IF(J.GE.I) THEN
                               P(K,I,J)=Pdij(I,J,AL,CN,dE,w,Maxi)
                           ELSE
                                P(K,I,J)=Puij(w,AL,CN,I,J,T,dE,Maxi)
                           ENDIF
                        ENDDO
                ENDDO

                DO I=1, Maxi
                        C(I)=0.0D0
                        DO J=1, Maxi
                                C(I)=C(I)+P(K,J,I)
                        ENDDO
                ENDDO

                DO I=1, Maxi
                IF(C(I).NE.0.0d0) THEN
                        DO J=1, Maxi
                                P(K,J,I)=P(K,J,I)/C(I)
                        ENDDO
                ENDIF
                ENDDO
        ENDDO

        RETURN
        END

