!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!CCC
!CCC Name       : FUNCTION Pdij 
!CCC Purpose    : To compute down energy transfer probability function 
!CCC 
!CCC Authors    : Thanh Lam Nguyen (thanhlamnguyen08@gmail.com)
!CCC              John F. Stanton  (jfstanton@gmail.com)
!CCC
!CCC Date       : Apr. 18, 2014
!CCC Where      : The University of Texas at Austin
!CCC
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

!C=====================================================================================
!CCC     Computing a energy-transfer probable function
!CCC     using a simple exponential model
!C=====================================================================================

        Double precision FUNCTION Pdij(I,J,AL,CN,dE,w,Maxi)

        IMPLICIT NONE

!!!  J          : Index of bin; starting at J 
!!!  I          : Index of bin; ending at I
!!!  AL         : down energy transfer (cm-1)
!!!  CN         : normalization factor
!!!  dE         : energy bin (grain energy)
!!!  w          : density of states for considered intermediate
!!! Maxi        : maximum index in master-equation

        Integer I, J, Maxi

        Double precision CN(Maxi), w(Maxi)
        Double precision EI, EJ, AL, dE

!CCCCCCCCCC Down energy transfer from EJ to EI with EJ >= EI

        EI=dE*(I-1)
        EJ=dE*(J-1)

       IF(w(J)*w(I).EQ.0.0d0) THEN
         Pdij = 0.0d0
       ELSE IF(CN(J).NE.0.0d0) THEN
         Pdij=exp((EI-EJ)/AL)/CN(J)
       ELSE
         Pdij = 0.0d0
       ENDIF

        RETURN
        END

